#include "../common/cleanup.h"
#include "../common/const.h"
#include "../common/global.h"
#include "../common/tls.h"

#include <csignal>
#include <cstring>
#include <iostream>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>
#include <wolfssl/options.h>
#include <wolfssl/ssl.h>

WOLFSSL_CTX *ctx;
int sockfd;

int main() {
  std::signal(SIGTERM, safe_exit);
  std::signal(SIGINT, safe_exit);
  std::signal(SIGQUIT, safe_exit);
  std::signal(SIGHUP, safe_exit);

  // error signal handlers
  signal(SIGSEGV, safe_exit);

  // Initialize wolfSSL library
  wolfSSL_Init();

  ctx = create_context(true);

  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    std::cerr << "Socket creation failed: " << strerror(errno) << std::endl;
    return 1;
  }

  std::cout << "Socket created successfully (fd: " << sockfd << ")"
            << std::endl;

  sockaddr_in addr{};
  addr.sin_family = AF_INET;
  addr.sin_port = htons(PORT);
  addr.sin_addr.s_addr = INADDR_ANY;

  if (bind(sockfd, (sockaddr *)&addr, sizeof(addr)) != 0) {
    std::cerr << "Bind failed: " << strerror(errno) << std::endl;
    return 1;
  }

  listen(sockfd, 1);

  while (true) {
    sockaddr_in client_addr{};
    socklen_t len = sizeof(client_addr);
    int client_fd = accept(sockfd, (sockaddr *)&client_addr, &len);

    WOLFSSL *ssl = wolfSSL_new(ctx);
    wolfSSL_set_fd(ssl, client_fd);

    if (wolfSSL_accept(ssl) != WOLFSSL_SUCCESS) {
      std::cerr << "wolfSSL_accept failed" << std::endl;
      wolfSSL_free(ssl);
      close(client_fd);
      continue;
    }

    handle_connection(ssl);

    wolfSSL_shutdown(ssl);
    wolfSSL_free(ssl);
    close(client_fd);
  }

  wolfSSL_CTX_free(ctx);
  wolfSSL_Cleanup();
  close(sockfd);
  return 0;
}
